@@ -30,4 +30,15 @@ check_pqs <- function(...) {
   if (different_lengths(...)) {
     stop(gettextf("%s must be the same length", toString(sQuote(vars, "'"))))
   }
-}
\ No newline at end of file
+}
+
+#' Make a list of balanced product indexes over time
+#' @noRd
+balance_products <- function(product, period) {
+  ux <- unique(product)
+  product <- split(product, period)
+  if (duplicate_products(product)) {
+    warning("there are duplicated period-product pairs")
+  }
+  lapply(product, \(x) match(ux, x, incomparables = NA))
+}
