@@ -19,8 +19,8 @@ extended_mean_ <- function(r, s) {
   if (not_finite_scalar(s)) {
     stop("'s' must be a finite length 1 numeric")
   }
-  
-  function(x, m, tol = .Machine$double.eps^0.5) {
+
+  function(x, m, tol) {
     res <- rdiff(x, m, r) / rdiff(x, m, s)
     res[abs(x - m) <= tol] <- m^(r - s)
     res
@@ -45,34 +45,24 @@ extended_mean_ <- function(r, s) {
 #' \preformatted{nested_mean(r1, r2, t)(x, w1, w2) ==
 #'   generalized_mean(s)(x, v(x, w1, w2))}
 #'
-#' This generalizes the result for turning a geometric mean into an arithmetic
-#' mean (and vice versa) in section 4.2 of Balk (2008), and a Fisher mean into
-#' an arithmetic mean in section 6 of Reinsdorf et al. (2002), although these
-#' are usually the most important cases. See Martin (2021) for details.
-#' `nested_transmute2()` takes a slightly different approach than
-#' `nested_transmute()`, generalizing the van IJzeren arithmetic
-#' decomposition for the Fisher index (Balk, 2008, section 4.2.2) using the
-#' approach by Martin (2021), although in most cases the results are broadly
-#' similar.
-#'
 #' Transmuting weights returns a value that is the same length as \code{x},
 #' so any missing values in \code{x} or the weights will return \code{NA}.
-#' Unless all values are \code{NA}, however, the result for will still satisfy
+#' Unless all values are \code{NA}, however, the result will still satisfy
 #' the above identities when \code{na.rm = TRUE}.
 #'
 #' @inheritParams nested_mean
 #' @param r,s A finite number giving the order of the generalized mean. See
-#' details.
+#'   details.
 #'
 #' @returns
 #' `transmute_weights()` returns a function:
 #'
-#' \preformatted{function(x, w = NULL){...}}
+#' \preformatted{function(x, w = NULL, tol = .Machine$double.eps^0.5){...}}
 #'
 #' `nested_transmute()` and `nested_transmute2()` similarly return a
 #' function:
 #'
-#' \preformatted{function(x, w1 = NULL, w2 = NULL){...}}
+#' \preformatted{function(x, w1 = NULL, w2 = NULL, tol = .Machine$double.eps^0.5){...}}
 #'
 #' @seealso
 #' [generalized_mean()] for the generalized mean and [nested_mean()] for the
@@ -87,45 +77,20 @@ extended_mean_ <- function(r, s) {
 #' [grouped()] to make these functions operate on grouped data.
 #'
 #' @references
-#' Balk, B. M. (2008). *Price and Quantity Index Numbers*.
-#' Cambridge University Press.
-#'
-#' Martin, S. (2021). A note on general decompositions for price indexes.
-#' *Prices Analytical Series*, Statistics Canada catalogue no. 62F0014M.
-#' Statistics Canada, Ottawa.
-#'
-#' Reinsdorf, M. B., Diewert, W. E., and Ehemann, C. (2002). Additive
-#' decompositions for Fisher, Törnqvist and geometric mean indexes.
-#' *Journal of Economic and Social Measurement*, 28(1-2):51--61.
-#'
-#' Sydsaeter, K., Strom, A., and Berck, P. (2005). *Economists'
-#' Mathematical Manual* (4th edition). Springer.
+#' See `vignette("decomposing-indexes")` for more details.
 #'
 #' @examples
 #' x <- 1:3
-#' y <- 4:6
 #' w <- 3:1
 #'
-#' #---- Transforming generalized means ----
-#'
 #' # Calculate the geometric mean as an arithmetic mean and
-#' # harmonic mean by transmuting the weights
+#' # harmonic mean by transmuting the weights.
 #'
 #' geometric_mean(x)
 #' arithmetic_mean(x, transmute_weights(0, 1)(x))
 #' harmonic_mean(x, transmute_weights(0, -1)(x))
 #'
-#' # Transmuting the weights for a harmonic mean into those
-#' # for an arithmetic mean is the same as using weights w / x
-#'
-#' all.equal(transmute_weights(-1, 1)(x, w), scale_weights(w / x))
-#' 
-#' # Transmuting the weights for an arithmetic mean into those
-#' # for a harmonic mean is the same as using weights w * x
-#'
-#' all.equal(transmute_weights(1, -1)(x, w), scale_weights(w * x))
-#'
-#' # Works for nested means, too
+#' # Works for nested means, too.
 #'
 #' w1 <- 3:1
 #' w2 <- 1:3
@@ -134,55 +99,6 @@ extended_mean_ <- function(r, s) {
 #'
 #' arithmetic_mean(x, nested_transmute(0, c(1, -1), 1)(x, w1, w2))
 #' arithmetic_mean(x, nested_transmute2(0, c(1, -1), 1)(x, w1, w2))
-#'
-#' # Note that nested_transmute() has an invariance property
-#' # not shared by nested_transmute2()
-#'
-#' all.equal(
-#'   nested_transmute(0, c(1, -1), 1)(x, w1, w2),
-#'   transmute_weights(2, 1)(
-#'     x, nested_transmute(0, c(1, -1), 2)(x, w1, w2)
-#'   )
-#' )
-#'
-#' all.equal(
-#'   nested_transmute2(0, c(1, -1), 1)(x, w1, w2),
-#'   transmute_weights(2, 1)(
-#'     x, nested_transmute2(0, c(1, -1), 2)(x, w1, w2)
-#'   )
-#' )
-#' 
-#' #---- Monotonicity ----
-#' 
-#' # Transmuted weights increase when x is small and decrease
-#' # when x is large if r < s
-#' 
-#' transmute_weights(0, 1)(x, w) > scale_weights(w)
-#' 
-#' # The opposite happens when r > s
-#' 
-#' transmute_weights(1, 0)(x, w) > scale_weights(w)
-#'
-#' #---- Percent-change contributions ----
-#'
-#' # Transmuted weights can be used to calculate percent-change
-#' # contributions for, e.g., a geometric price index
-#'
-#' transmute_weights(0, 1)(x) * (x - 1)
-#' geometric_contributions(x) # the more convenient way
-#'
-#' #---- Basket representation of a price index ----
-#'
-#' # Any generalized-mean index can be represented as a basket-style
-#' # index by transmuting the weights, which is how some authors
-#' # define a price index (e.g., Sydsaeter et al., 2005, p. 174)
-#'
-#' p1 <- 2:6
-#' p0 <- 1:5
-#'
-#' qs <- transmute_weights(-1, 1)(p1 / p0) / p0
-#' all.equal(harmonic_mean(p1 / p0), sum(p1 * qs) / sum(p0 * qs))
-#'
 #' @family weights functions
 #' @export
 transmute_weights <- function(r, s) {
@@ -191,7 +107,7 @@ transmute_weights <- function(r, s) {
   gen_mean <- generalized_mean(r)
   pow_ext_mean <- extended_mean_(r, s)
 
-  function(x, w = NULL) {
+  function(x, w = NULL, tol = .Machine$double.eps^0.5) {
     if (r == s) {
       if (is.null(w)) {
         w <- rep.int(1, length(x))
@@ -202,12 +118,15 @@ transmute_weights <- function(r, s) {
       w[is.na(x)] <- NA_real_
       scale_weights(w)
     } else {
+      if (length(tol) != 1L && length(tol) != length(x)) {
+        stop("'tol' must be either length 1 or the same length as 'x'")
+      }
       m <- gen_mean(x, w, na.rm = TRUE)
       if (is.null(w)) {
-        v <- pow_ext_mean(x, m)
+        v <- pow_ext_mean(x, m, tol)
         attributes(v) <- NULL
       } else {
-        v <- w * pow_ext_mean(x, m)
+        v <- w * pow_ext_mean(x, m, tol)
         attributes(v) <- attributes(w)
       }
       scale_weights(v)
@@ -234,14 +153,14 @@ nested_transmute <- function(r1, r2, s, t = c(1, 1)) {
     stop("'t' must be a pair of numeric values")
   }
 
-  function(x, w1 = NULL, w2 = NULL) {
+  function(x, w1 = NULL, w2 = NULL, tol = .Machine$double.eps^0.5) {
     if (is.na(t[1L]) && !is.na(t[2L])) {
-      w <- r_weights2(x, w2)
+      w <- r_weights2(x, w2, tol)
     } else if (!is.na(t[1L]) && is.na(t[2L])) {
-      w <- r_weights1(x, w1)
+      w <- r_weights1(x, w1, tol)
     } else {
-      v1 <- r_weights1(x, w1)
-      v2 <- r_weights2(x, w2)
+      v1 <- r_weights1(x, w1, tol)
+      v2 <- r_weights2(x, w2, tol)
       # The calculation is wrong if NAs in w1 or w2 propagate.
       if (anyNA(w1)) {
         v1[is.na(v1) & !is.na(v2)] <- 0
@@ -251,7 +170,7 @@ nested_transmute <- function(r1, r2, s, t = c(1, 1)) {
       }
       w <- t[1L] * v1 + t[2L] * v2
     }
-    s_weights(x, w)
+    s_weights(x, w, tol)
   }
 }
 
@@ -276,16 +195,16 @@ nested_transmute2 <- function(r1, r2, s, t = c(1, 1)) {
     stop("'t' must be a pair of numeric values")
   }
 
-  function(x, w1 = NULL, w2 = NULL) {
+  function(x, w1 = NULL, w2 = NULL, tol = .Machine$double.eps^0.5) {
     m <- c(mean1(x, w1, na.rm = TRUE), mean2(x, w2, na.rm = TRUE))
-    v <- s_weights(m, t)
+    v <- s_weights(m, t, tol)
     if (is.na(v[1L]) && !is.na(v[2L])) {
-      s_weights2(x, w2)
+      s_weights2(x, w2, tol)
     } else if (!is.na(v[1L]) && is.na(v[2L])) {
-      s_weights1(x, w1)
+      s_weights1(x, w1, tol)
     } else {
-      u1 <- s_weights1(x, w1)
-      u2 <- s_weights2(x, w2)
+      u1 <- s_weights1(x, w1, tol)
+      u2 <- s_weights2(x, w2, tol)
       # The calculation is wrong if NAs in w1 or w2 propagate.
       if (anyNA(w1)) {
         u1[is.na(u1) & !is.na(u2)] <- 0
@@ -319,7 +238,7 @@ nested_transmute2 <- function(r1, r2, s, t = c(1, 1)) {
 #'
 #' Factoring weights return a value that is the same length as \code{x},
 #' so any missing values in \code{x} or the weights will return \code{NA}.
-#' Unless all values are \code{NA}, however, the result for will still satisfy
+#' Unless all values are \code{NA}, however, the result will still satisfy
 #' the above identity when \code{na.rm = TRUE}.
 #'
 #' @inheritParams generalized_mean
@@ -346,18 +265,18 @@ nested_transmute2 <- function(r1, r2, s, t = c(1, 1)) {
 #' y <- 4:6
 #' w <- 3:1
 
-#' # Factor the harmonic mean by chaining the calculation
+#' # Factor the harmonic mean by chaining the calculation.
 #'
 #' harmonic_mean(x * y, w)
 #' harmonic_mean(x, w) * harmonic_mean(y, factor_weights(-1)(x, w))
 #'
-#' # The common case of an arithmetic mean
+#' # The common case of an arithmetic mean.
 #'
 #' arithmetic_mean(x * y, w)
 #' arithmetic_mean(x, w) * arithmetic_mean(y, update_weights(x, w))
 #'
 #' # In cases where x and y have the same order, Chebyshev's
-#' # inequality implies that the chained calculation is too small
+#' # inequality implies that the chained calculation is too small.
 #'
 #' arithmetic_mean(x * y, w) >
 #'   arithmetic_mean(x, w) * arithmetic_mean(y, w)
@@ -414,6 +333,8 @@ update_weights <- factor_weights(1)
 #' @examples
 #' scale_weights(1:5)
 #'
+#' scale_weights(c(1:5, NA))
+#'
 #' @family weights functions
 #' @export
 scale_weights <- function(x) {
